@@ -3,19 +3,26 @@
 These can be used for filtering specific errors.
 """
 
-from typing import List
+from typing import Dict, List
 from typing_extensions import Final
 
 
 # All created error codes are implicitly stored in this list.
 all_error_codes = []  # type: List[ErrorCode]
 
+error_codes = {}  # type: Dict[str, ErrorCode]
+
 
 class ErrorCode:
-    def __init__(self, code: str, description: str, category: str) -> None:
+    def __init__(self, code: str,
+                 description: str,
+                 category: str,
+                 default_enabled: bool = True) -> None:
         self.code = code
         self.description = description
         self.category = category
+        self.default_enabled = default_enabled
+        error_codes[code] = self
 
     def __str__(self) -> str:
         return '<ErrorCode {}>'.format(self.code)
@@ -77,8 +84,16 @@ ABSTRACT = ErrorCode(
     'General')  # type: Final
 VALID_NEWTYPE = ErrorCode(
     'valid-newtype', "Check that argument 2 to NewType is valid", 'General')  # type: Final
+STRING_FORMATTING = ErrorCode(
+    'str-format', "Check that string formatting/interpolation is type-safe",
+    'General')  # type: Final
+STR_BYTES_PY3 = ErrorCode(
+    'str-bytes-safe', "Warn about dangerous coercions related to bytes and string types",
+    'General')  # type: Final
+EXIT_RETURN = ErrorCode(
+    'exit-return', "Warn about too general return type for '__exit__'", 'General')  # type: Final
 
-# These error codes aren't enable by default.
+# These error codes aren't enabled by default.
 NO_UNTYPED_DEF = ErrorCode(
     'no-untyped-def', "Check that every function has an annotation", 'General')  # type: Final
 NO_UNTYPED_CALL = ErrorCode(
@@ -95,6 +110,16 @@ NO_ANY_UNIMPORTED = ErrorCode(
 NO_ANY_RETURN = ErrorCode(
     'no-any-return', 'Reject returning value with "Any" type if return type is not "Any"',
     'General')  # type: Final
+UNREACHABLE = ErrorCode(
+    'unreachable', "Warn about unreachable statements or expressions", 'General')  # type: Final
+REDUNDANT_EXPR = ErrorCode(
+    'redundant-expr',
+    "Warn about redundant expressions",
+    'General',
+    default_enabled=False)  # type: Final
+NAME_MATCH = ErrorCode(
+    'name-match', "Check that type definition has consistent naming", 'General')  # type: Final
+
 
 # Syntax errors are often blocking.
 SYNTAX = ErrorCode(
@@ -102,4 +127,4 @@ SYNTAX = ErrorCode(
 
 # This is a catch-all for remaining uncategorized errors.
 MISC = ErrorCode(
-    'misc', "Miscenallenous other checks", 'General')  # type: Final
+    'misc', "Miscellaneous other checks", 'General')  # type: Final
