@@ -109,7 +109,7 @@ class IPCBase:
                 assert err == 0, err
                 assert bytes_written == len(data)
             except WindowsError as e:
-                raise IPCException("Failed to write with error: {}".format(e.winerror))
+                raise IPCException("Failed to write with error: {}".format(e.winerror)) from e
         else:
             self.connection.sendall(data)
             self.connection.shutdown(socket.SHUT_WR)
@@ -131,11 +131,11 @@ class IPCClient(IPCBase):
             timeout = int(self.timeout * 1000) if self.timeout else _winapi.NMPWAIT_WAIT_FOREVER
             try:
                 _winapi.WaitNamedPipe(self.name, timeout)
-            except FileNotFoundError:
-                raise IPCException("The NamedPipe at {} was not found.".format(self.name))
+            except FileNotFoundError as e:
+                raise IPCException("The NamedPipe at {} was not found.".format(self.name)) from e
             except WindowsError as e:
                 if e.winerror == _winapi.ERROR_SEM_TIMEOUT:
-                    raise IPCException("Timed out waiting for connection.")
+                    raise IPCException("Timed out waiting for connection.") from e
                 else:
                     raise
             try:
@@ -150,7 +150,7 @@ class IPCClient(IPCBase):
                 )
             except WindowsError as e:
                 if e.winerror == _winapi.ERROR_PIPE_BUSY:
-                    raise IPCException("The connection is busy.")
+                    raise IPCException("The connection is busy.") from e
                 else:
                     raise
             _winapi.SetNamedPipeHandleState(self.connection,
@@ -169,9 +169,8 @@ class IPCClient(IPCBase):
                  exc_ty: 'Optional[Type[BaseException]]' = None,
                  exc_val: Optional[BaseException] = None,
                  exc_tb: Optional[TracebackType] = None,
-                 ) -> bool:
+                 ) -> None:
         self.close()
-        return False
 
 
 class IPCServer(IPCBase):
@@ -238,15 +237,15 @@ class IPCServer(IPCBase):
         else:
             try:
                 self.connection, _ = self.sock.accept()
-            except socket.timeout:
-                raise IPCException('The socket timed out')
+            except socket.timeout as e:
+                raise IPCException('The socket timed out') from e
         return self
 
     def __exit__(self,
                  exc_ty: 'Optional[Type[BaseException]]' = None,
                  exc_val: Optional[BaseException] = None,
                  exc_tb: Optional[TracebackType] = None,
-                 ) -> bool:
+                 ) -> None:
         if sys.platform == 'win32':
             try:
                 # Wait for the client to finish reading the last write before disconnecting
@@ -257,7 +256,6 @@ class IPCServer(IPCBase):
                 DisconnectNamedPipe(self.connection)
         else:
             self.close()
-        return False
 
     def cleanup(self) -> None:
         if sys.platform == 'win32':
