@@ -1,16 +1,16 @@
 from typing import Sequence
 
 from mypy.types import (
-    Type, UnboundType, AnyType, NoneType, TupleType, TypedDictType,
+    Type, TypeGuardType, UnboundType, AnyType, NoneType, TupleType, TypedDictType,
     UnionType, CallableType, TypeVarType, Instance, TypeVisitor, ErasedType,
     Overloaded, PartialType, DeletedType, UninhabitedType, TypeType, LiteralType,
-    ProperType, get_proper_type
-)
+    ProperType, get_proper_type, TypeAliasType)
 from mypy.typeops import tuple_fallback, make_simplified_union
 
 
 def is_same_type(left: Type, right: Type) -> bool:
     """Is 'left' the same type as 'right'?"""
+
     left = get_proper_type(left)
     right = get_proper_type(right)
 
@@ -85,6 +85,13 @@ class SameTypeVisitor(TypeVisitor[bool]):
                 is_same_types(left.args, self.right.args) and
                 left.last_known_value == self.right.last_known_value)
 
+    def visit_type_alias_type(self, left: TypeAliasType) -> bool:
+        # Similar to protocols, two aliases with the same targets return False here,
+        # but both is_subtype(t, s) and is_subtype(s, t) return True.
+        return (isinstance(self.right, TypeAliasType) and
+                left.alias == self.right.alias and
+                is_same_types(left.args, self.right.args))
+
     def visit_type_var(self, left: TypeVarType) -> bool:
         return (isinstance(self.right, TypeVarType) and
                 left.id == self.right.id)
@@ -144,6 +151,12 @@ class SameTypeVisitor(TypeVisitor[bool]):
         else:
             return False
 
+    def visit_type_guard_type(self, left: TypeGuardType) -> bool:
+        if isinstance(self.right, TypeGuardType):
+            return is_same_type(left.type_guard, self.right.type_guard)
+        else:
+            return False
+
     def visit_overloaded(self, left: Overloaded) -> bool:
         if isinstance(self.right, Overloaded):
             return is_same_types(left.items(), self.right.items())
