@@ -7,8 +7,8 @@ from typing import List, Tuple, Optional, Union, cast
 
 from mypy.nodes import (
     Expression, Context, TypeInfo, AssignmentStmt, NameExpr, CallExpr, RefExpr, StrExpr,
-    UnicodeExpr, TupleExpr, ListExpr, DictExpr, Var, SymbolTableNode, GDEF, MDEF, ARG_POS,
-    EnumCallExpr
+    UnicodeExpr, TupleExpr, ListExpr, DictExpr, Var, SymbolTableNode, MDEF, ARG_POS,
+    ARG_NAMED, EnumCallExpr, MemberExpr
 )
 from mypy.semanal_shared import SemanticAnalyzerInterface
 from mypy.options import Options
@@ -19,24 +19,29 @@ class EnumCallAnalyzer:
         self.options = options
         self.api = api
 
-    def process_enum_call(self, s: AssignmentStmt, is_func_scope: bool) -> None:
-        """Check if s defines an Enum; if yes, store the definition in symbol table."""
-        if len(s.lvalues) != 1 or not isinstance(s.lvalues[0], NameExpr):
-            return
+    def process_enum_call(self, s: AssignmentStmt, is_func_scope: bool) -> bool:
+        """Check if s defines an Enum; if yes, store the definition in symbol table.
+
+        Return True if this looks like an Enum definition (but maybe with errors),
+        otherwise return False.
+        """
+        if len(s.lvalues) != 1 or not isinstance(s.lvalues[0], (NameExpr, MemberExpr)):
+            return False
         lvalue = s.lvalues[0]
         name = lvalue.name
         enum_call = self.check_enum_call(s.rvalue, name, is_func_scope)
         if enum_call is None:
-            return
+            return False
+        if isinstance(lvalue, MemberExpr):
+            self.fail("Enum type as attribute is not supported", lvalue)
+            return False
         # Yes, it's a valid Enum definition. Add it to the symbol table.
-        node = self.api.lookup(name, s)
-        if node:
-            node.kind = GDEF   # TODO locally defined Enum
-            node.node = enum_call
+        self.api.add_symbol(name, enum_call, s)
+        return True
 
     def check_enum_call(self,
                         node: Expression,
-                        var_name: Optional[str],
+                        var_name: str,
                         is_func_scope: bool) -> Optional[TypeInfo]:
         """Check if a call defines an Enum.
 
@@ -62,55 +67,75 @@ class EnumCallAnalyzer:
         items, values, ok = self.parse_enum_call_args(call, fullname.split('.')[-1])
         if not ok:
             # Error. Construct dummy return value.
-            return self.build_enum_call_typeinfo('Enum', [], fullname)
-        name = cast(Union[StrExpr, UnicodeExpr], call.args[0]).value
-        if name != var_name or is_func_scope:
-            # Give it a unique name derived from the line number.
-            name += '@' + str(call.line)
-        info = self.build_enum_call_typeinfo(name, items, fullname)
-        # Store it as a global just in case it would remain anonymous.
-        # (Or in the nearest class if there is one.)
-        stnode = SymbolTableNode(GDEF, info)
-        self.api.add_symbol_table_node(name, stnode)
+            info = self.build_enum_call_typeinfo(var_name, [], fullname, node.line)
+        else:
+            name = cast(Union[StrExpr, UnicodeExpr], call.args[0]).value
+            if name != var_name or is_func_scope:
+                # Give it a unique name derived from the line number.
+                name += '@' + str(call.line)
+            info = self.build_enum_call_typeinfo(name, items, fullname, call.line)
+            # Store generated TypeInfo under both names, see semanal_namedtuple for more details.
+            if name != var_name or is_func_scope:
+                self.api.add_symbol_skip_local(name, info)
         call.analyzed = EnumCallExpr(info, items, values)
         call.analyzed.set_line(call.line, call.column)
+        info.line = node.line
         return info
 
-    def build_enum_call_typeinfo(self, name: str, items: List[str], fullname: str) -> TypeInfo:
+    def build_enum_call_typeinfo(self, name: str, items: List[str], fullname: str,
+                                 line: int) -> TypeInfo:
         base = self.api.named_type_or_none(fullname)
         assert base is not None
-        info = self.api.basic_new_typeinfo(name, base)
+        info = self.api.basic_new_typeinfo(name, base, line)
         info.metaclass_type = info.calculate_metaclass_type()
         info.is_enum = True
         for item in items:
             var = Var(item)
             var.info = info
             var.is_property = True
-            var._fullname = '{}.{}'.format(self.api.qualified_name(name), item)
+            var._fullname = '{}.{}'.format(info.fullname, item)
             info.names[item] = SymbolTableNode(MDEF, var)
         return info
 
     def parse_enum_call_args(self, call: CallExpr,
                              class_name: str) -> Tuple[List[str],
                                                        List[Optional[Expression]], bool]:
+        """Parse arguments of an Enum call.
+
+        Return a tuple of fields, values, was there an error.
+        """
         args = call.args
+        if not all([arg_kind in [ARG_POS, ARG_NAMED] for arg_kind in call.arg_kinds]):
+            return self.fail_enum_call_arg("Unexpected arguments to %s()" % class_name, call)
         if len(args) < 2:
             return self.fail_enum_call_arg("Too few arguments for %s()" % class_name, call)
-        if len(args) > 2:
+        if len(args) > 6:
             return self.fail_enum_call_arg("Too many arguments for %s()" % class_name, call)
-        if call.arg_kinds != [ARG_POS, ARG_POS]:
-            return self.fail_enum_call_arg("Unexpected arguments to %s()" % class_name, call)
-        if not isinstance(args[0], (StrExpr, UnicodeExpr)):
+        valid_name = [None, 'value', 'names', 'module', 'qualname', 'type', 'start']
+        for arg_name in call.arg_names:
+            if arg_name not in valid_name:
+                self.fail_enum_call_arg('Unexpected keyword argument "{}"'.format(arg_name), call)
+        value, names = None, None
+        for arg_name, arg in zip(call.arg_names, args):
+            if arg_name == 'value':
+                value = arg
+            if arg_name == 'names':
+                names = arg
+        if value is None:
+            value = args[0]
+        if names is None:
+            names = args[1]
+        if not isinstance(value, (StrExpr, UnicodeExpr)):
             return self.fail_enum_call_arg(
                 "%s() expects a string literal as the first argument" % class_name, call)
         items = []
         values = []  # type: List[Optional[Expression]]
-        if isinstance(args[1], (StrExpr, UnicodeExpr)):
-            fields = args[1].value
+        if isinstance(names, (StrExpr, UnicodeExpr)):
+            fields = names.value
             for field in fields.replace(',', ' ').split():
                 items.append(field)
-        elif isinstance(args[1], (TupleExpr, ListExpr)):
-            seq_items = args[1].items
+        elif isinstance(names, (TupleExpr, ListExpr)):
+            seq_items = names.items
             if all(isinstance(seq_item, (StrExpr, UnicodeExpr)) for seq_item in seq_items):
                 items = [cast(Union[StrExpr, UnicodeExpr], seq_item).value
                          for seq_item in seq_items]
@@ -129,8 +154,8 @@ class EnumCallAnalyzer:
                     "%s() with tuple or list expects strings or (name, value) pairs" %
                     class_name,
                     call)
-        elif isinstance(args[1], DictExpr):
-            for key, value in args[1].items:
+        elif isinstance(names, DictExpr):
+            for key, value in names.items:
                 if not isinstance(key, (StrExpr, UnicodeExpr)):
                     return self.fail_enum_call_arg(
                         "%s() with dict literal requires string literals" % class_name, call)
