@@ -42,23 +42,27 @@ class NewTypeAnalyzer:
         The logic in this function mostly copies the logic for visit_class_def()
         with a single (non-Generic) base.
         """
-        name, call = self.analyze_newtype_declaration(s)
-        if name is None or call is None:
+        var_name, call = self.analyze_newtype_declaration(s)
+        if var_name is None or call is None:
             return False
+        name = var_name
         # OK, now we know this is a NewType. But the base type may be not ready yet,
         # add placeholder as we do for ClassDef.
 
+        if self.api.is_func_scope():
+            name += '@' + str(s.line)
         fullname = self.api.qualified_name(name)
+
         if (not call.analyzed or
                 isinstance(call.analyzed, NewTypeExpr) and not call.analyzed.info):
             # Start from labeling this as a future class, as we do for normal ClassDefs.
             placeholder = PlaceholderNode(fullname, s, s.line, becomes_typeinfo=True)
-            self.api.add_symbol(name, placeholder, s, can_defer=False)
+            self.api.add_symbol(var_name, placeholder, s, can_defer=False)
 
-        old_type, should_defer = self.check_newtype_args(name, call, s)
+        old_type, should_defer = self.check_newtype_args(var_name, call, s)
         old_type = get_proper_type(old_type)
         if not call.analyzed:
-            call.analyzed = NewTypeExpr(name, old_type, line=call.line, column=call.column)
+            call.analyzed = NewTypeExpr(var_name, old_type, line=call.line, column=call.column)
         if old_type is None:
             if should_defer:
                 # Base type is not ready.
@@ -68,12 +72,12 @@ class NewTypeAnalyzer:
         # Create the corresponding class definition if the aliased type is subtypeable
         if isinstance(old_type, TupleType):
             newtype_class_info = self.build_newtype_typeinfo(name, old_type,
-                                                             old_type.partial_fallback)
+                                                             old_type.partial_fallback, s.line)
             newtype_class_info.tuple_type = old_type
         elif isinstance(old_type, Instance):
             if old_type.type.is_protocol:
                 self.fail("NewType cannot be used with protocol classes", s)
-            newtype_class_info = self.build_newtype_typeinfo(name, old_type, old_type)
+            newtype_class_info = self.build_newtype_typeinfo(name, old_type, old_type, s.line)
         else:
             if old_type is not None:
                 message = "Argument 2 to NewType(...) must be subclassable (got {})"
@@ -81,7 +85,7 @@ class NewTypeAnalyzer:
             # Otherwise the error was already reported.
             old_type = AnyType(TypeOfAny.from_error)
             object_type = self.api.named_type('__builtins__.object')
-            newtype_class_info = self.build_newtype_typeinfo(name, old_type, object_type)
+            newtype_class_info = self.build_newtype_typeinfo(name, old_type, object_type, s.line)
             newtype_class_info.fallback_to_any = True
 
         check_for_explicit_any(old_type, self.options, self.api.is_typeshed_stub_file, self.msg,
@@ -98,7 +102,9 @@ class NewTypeAnalyzer:
             call.analyzed.info = newtype_class_info
         else:
             call.analyzed.info.bases = newtype_class_info.bases
-        self.api.add_symbol(name, call.analyzed.info, s)
+        self.api.add_symbol(var_name, call.analyzed.info, s)
+        if self.api.is_func_scope():
+            self.api.add_symbol_skip_local(name, call.analyzed.info)
         newtype_class_info.line = s.line
         return True
 
@@ -121,7 +127,7 @@ class NewTypeAnalyzer:
             # Give a better error message than generic "Name already defined".
             if (existing and
                     not isinstance(existing.node, PlaceholderNode) and not s.rvalue.analyzed):
-                self.fail("Cannot redefine '%s' as a NewType" % name, s)
+                self.fail('Cannot redefine "%s" as a NewType' % name, s)
 
             # This dummy NewTypeExpr marks the call as sufficiently analyzed; it will be
             # overwritten later with a fully complete NewTypeExpr if there are no other
@@ -147,7 +153,7 @@ class NewTypeAnalyzer:
             self.fail("Argument 1 to NewType(...) must be a string literal", context)
             has_failed = True
         elif args[0].value != name:
-            msg = "String argument 1 '{}' to NewType(...) does not match variable name '{}'"
+            msg = 'String argument 1 "{}" to NewType(...) does not match variable name "{}"'
             self.fail(msg.format(args[0].value, name), context)
             has_failed = True
 
@@ -175,8 +181,9 @@ class NewTypeAnalyzer:
 
         return None if has_failed else old_type, should_defer
 
-    def build_newtype_typeinfo(self, name: str, old_type: Type, base_type: Instance) -> TypeInfo:
-        info = self.api.basic_new_typeinfo(name, base_type)
+    def build_newtype_typeinfo(self, name: str, old_type: Type, base_type: Instance,
+                               line: int) -> TypeInfo:
+        info = self.api.basic_new_typeinfo(name, base_type, line)
         info.is_newtype = True
 
         # Add __init__ method
@@ -191,7 +198,7 @@ class NewTypeAnalyzer:
             name=name)
         init_func = FuncDef('__init__', args, Block([]), typ=signature)
         init_func.info = info
-        init_func._fullname = self.api.qualified_name(name) + '.__init__'
+        init_func._fullname = info.fullname + '.__init__'
         info.names['__init__'] = SymbolTableNode(MDEF, init_func)
 
         return info
