@@ -57,7 +57,7 @@ from mypy.nodes import (
     FuncBase, OverloadedFuncDef, FuncItem, MypyFile, UNBOUND_IMPORTED
 )
 from mypy.types import (
-    Type, TypeVisitor, UnboundType, AnyType, NoneType, UninhabitedType,
+    Type, TypeGuardType, TypeVisitor, UnboundType, AnyType, NoneType, UninhabitedType,
     ErasedType, DeletedType, Instance, TypeVarType, CallableType, TupleType, TypedDictType,
     UnionType, Overloaded, PartialType, TypeType, LiteralType, TypeAliasType
 )
@@ -335,6 +335,9 @@ class SnapshotTypeVisitor(TypeVisitor[SnapshotItem]):
         normalized = tuple(sorted(items))
         return ('UnionType', normalized)
 
+    def visit_type_guard_type(self, typ: TypeGuardType) -> SnapshotItem:
+        return ('TypeGuardType', snapshot_type(typ.type_guard))
+
     def visit_overloaded(self, typ: Overloaded) -> SnapshotItem:
         return ('Overloaded', snapshot_types(typ.items()))
 
