@@ -59,7 +59,7 @@ from mypy.types import (
     Type, SyntheticTypeVisitor, Instance, AnyType, NoneType, CallableType, ErasedType, DeletedType,
     TupleType, TypeType, TypeVarType, TypedDictType, UnboundType, UninhabitedType, UnionType,
     Overloaded, TypeVarDef, TypeList, CallableArgument, EllipsisType, StarType, LiteralType,
-    RawExpressionType, PartialType, PlaceholderType, TypeAliasType
+    RawExpressionType, PartialType, PlaceholderType, TypeAliasType, TypeGuardType
 )
 from mypy.util import get_prefix, replace_object_state
 from mypy.typestate import TypeState
@@ -370,9 +370,10 @@ class TypeReplaceVisitor(SyntheticTypeVisitor[None]):
         if typ.fallback is not None:
             typ.fallback.accept(self)
         for tv in typ.variables:
-            tv.upper_bound.accept(self)
-            for value in tv.values:
-                value.accept(self)
+            if isinstance(tv, TypeVarDef):
+                tv.upper_bound.accept(self)
+                for value in tv.values:
+                    value.accept(self)
 
     def visit_overloaded(self, t: Overloaded) -> None:
         for item in t.items():
@@ -388,6 +389,9 @@ class TypeReplaceVisitor(SyntheticTypeVisitor[None]):
     def visit_deleted_type(self, typ: DeletedType) -> None:
         pass
 
+    def visit_type_guard_type(self, typ: TypeGuardType) -> None:
+        raise RuntimeError
+
     def visit_partial_type(self, typ: PartialType) -> None:
         raise RuntimeError
 
