@@ -112,9 +112,11 @@ This is module is tested using end-to-end fine-grained incremental mode
 test cases (test-data/unit/fine-grained*.test).
 """
 
+import os
+import sys
 import time
 from typing import (
-    Dict, List, Set, Tuple, Union, Optional, NamedTuple, Sequence
+    Dict, List, Set, Tuple, Union, Optional, NamedTuple, Sequence, Callable
 )
 from typing_extensions import Final
 
@@ -128,14 +130,16 @@ from mypy.checker import FineGrainedDeferredNode
 from mypy.errors import CompileError
 from mypy.nodes import (
     MypyFile, FuncDef, TypeInfo, SymbolNode, Decorator,
-    OverloadedFuncDef, SymbolTable
+    OverloadedFuncDef, SymbolTable, ImportFrom
 )
 from mypy.options import Options
 from mypy.fscache import FileSystemCache
 from mypy.server.astdiff import (
     snapshot_symbol_table, compare_symbol_table_snapshots, SnapshotItem
 )
-from mypy.semanal_main import semantic_analysis_for_scc, semantic_analysis_for_targets
+from mypy.semanal_main import (
+    semantic_analysis_for_scc, semantic_analysis_for_targets, core_modules
+)
 from mypy.server.astmerge import merge_asts
 from mypy.server.aststrip import strip_target, SavedAttributes
 from mypy.server.deps import get_dependencies_of_target, merge_dependencies
@@ -146,6 +150,8 @@ from mypy.typestate import TypeState
 
 MAX_ITER = 1000  # type: Final
 
+SENSITIVE_INTERNAL_MODULES = tuple(core_modules) + ("mypy_extensions", "typing_extensions")
+
 
 class FineGrainedBuildManager:
     def __init__(self, result: BuildResult) -> None:
@@ -226,15 +232,17 @@ class FineGrainedBuildManager:
             self.manager.log_fine_grained('previous targets with errors: %s' %
                              sorted(self.previous_targets_with_errors))
 
+        blocking_error = None
         if self.blocking_error:
             # Handle blocking errors first. We'll exit as soon as we find a
             # module that still has blocking errors.
             self.manager.log_fine_grained('existing blocker: %s' % self.blocking_error[0])
             changed_modules = dedupe_modules([self.blocking_error] + changed_modules)
+            blocking_error = self.blocking_error[0]
             self.blocking_error = None
 
         while True:
-            result = self.update_one(changed_modules, initial_set, removed_set)
+            result = self.update_one(changed_modules, initial_set, removed_set, blocking_error)
             changed_modules, (next_id, next_path), blocker_messages = result
 
             if blocker_messages is not None:
@@ -280,12 +288,21 @@ class FineGrainedBuildManager:
         self.previous_messages = self.manager.errors.new_messages()[:]
         return self.update(changed_modules, [])
 
+    def flush_cache(self) -> None:
+        """Flush AST cache.
+
+        This needs to be called after each increment, or file changes won't
+        be detected reliably.
+        """
+        self.manager.ast_cache.clear()
+
     def update_one(self,
                    changed_modules: List[Tuple[str, str]],
                    initial_set: Set[str],
-                   removed_set: Set[str]) -> Tuple[List[Tuple[str, str]],
-                                                   Tuple[str, str],
-                                                   Optional[List[str]]]:
+                   removed_set: Set[str],
+                   blocking_error: Optional[str]) -> Tuple[List[Tuple[str, str]],
+                                                           Tuple[str, str],
+                                                           Optional[List[str]]]:
         """Process a module from the list of changed modules.
 
         Returns:
@@ -297,9 +314,17 @@ class FineGrainedBuildManager:
         """
         t0 = time.time()
         next_id, next_path = changed_modules.pop(0)
-        if next_id not in self.previous_modules and next_id not in initial_set:
-            self.manager.log_fine_grained('skip %r (module not in import graph)' % next_id)
+
+        # If we have a module with a blocking error that is no longer
+        # in the import graph, we must skip it as otherwise we'll be
+        # stuck with the blocking error.
+        if (next_id == blocking_error
+                and next_id not in self.previous_modules
+                and next_id not in initial_set):
+            self.manager.log_fine_grained(
+                'skip %r (module with blocking error not in import graph)' % next_id)
             return changed_modules, (next_id, next_path), None
+
         result = self.update_module(next_id, next_path, next_id in removed_set)
         remaining, (next_id, next_path), blocker_messages = result
         changed_modules = [(id, path) for id, path in changed_modules
@@ -340,6 +365,12 @@ class FineGrainedBuildManager:
         self.manager.log_fine_grained('--- update single %r ---' % module)
         self.updated_modules.append(module)
 
+        # builtins and friends could potentially get triggered because
+        # of protocol stuff, but nothing good could possibly come from
+        # actually updating them.
+        if module in SENSITIVE_INTERNAL_MODULES:
+            return [], (module, path), None
+
         manager = self.manager
         previous_modules = self.previous_modules
         graph = self.graph
@@ -574,7 +605,6 @@ def update_module_isolated(module: str,
     state.parse_file()
     assert state.tree is not None, "file must be at least parsed"
     t0 = time.time()
-    # TODO: state.fix_suppressed_dependencies()?
     try:
         semantic_analysis_for_scc(graph, [state.id], manager.errors)
     except CompileError as err:
@@ -1108,3 +1138,87 @@ def target_from_node(module: str,
             return '%s.%s' % (node.info.fullname, node.name)
         else:
             return '%s.%s' % (module, node.name)
+
+
+if sys.platform != 'win32':
+    INIT_SUFFIXES = ('/__init__.py', '/__init__.pyi')  # type: Final
+else:
+    INIT_SUFFIXES = (
+        os.sep + '__init__.py',
+        os.sep + '__init__.pyi',
+        os.altsep + '__init__.py',
+        os.altsep + '__init__.pyi',
+    )  # type: Final
+
+
+def refresh_suppressed_submodules(
+        module: str,
+        path: Optional[str],
+        deps: Dict[str, Set[str]],
+        graph: Graph,
+        fscache: FileSystemCache,
+        refresh_file: Callable[[str, str], List[str]]) -> Optional[List[str]]:
+    """Look for submodules that are now suppressed in target package.
+
+    If a submodule a.b gets added, we need to mark it as suppressed
+    in modules that contain "from a import b". Previously we assumed
+    that 'a.b' is not a module but a regular name.
+
+    This is only relevant when following imports normally.
+
+    Args:
+        module: target package in which to look for submodules
+        path: path of the module
+        refresh_file: function that reads the AST of a module (returns error messages)
+
+    Return a list of errors from refresh_file() if it was called. If the
+    return value is None, we didn't call refresh_file().
+    """
+    messages = None
+    if path is None or not path.endswith(INIT_SUFFIXES):
+        # Only packages have submodules.
+        return None
+    # Find any submodules present in the directory.
+    pkgdir = os.path.dirname(path)
+    try:
+        entries = fscache.listdir(pkgdir)
+    except FileNotFoundError:
+        entries = []
+    for fnam in entries:
+        if (not fnam.endswith(('.py', '.pyi'))
+                or fnam.startswith("__init__.")
+                or fnam.count('.') != 1):
+            continue
+        shortname = fnam.split('.')[0]
+        submodule = module + '.' + shortname
+        trigger = make_trigger(submodule)
+
+        # We may be missing the required fine-grained deps.
+        ensure_deps_loaded(module, deps, graph)
+
+        if trigger in deps:
+            for dep in deps[trigger]:
+                # We can ignore <...> deps since a submodule can't trigger any.
+                state = graph.get(dep)
+                if not state:
+                    # Maybe it's a non-top-level target. We only care about the module.
+                    dep_module = module_prefix(graph, dep)
+                    if dep_module is not None:
+                        state = graph.get(dep_module)
+                if state:
+                    # Is the file may missing an AST in case it's read from cache?
+                    if state.tree is None:
+                        # Create AST for the file. This may produce some new errors
+                        # that we need to propagate.
+                        assert state.path is not None
+                        messages = refresh_file(state.id, state.path)
+                    tree = state.tree
+                    assert tree  # Will be fine, due to refresh_file() above
+                    for imp in tree.imports:
+                        if isinstance(imp, ImportFrom):
+                            if (imp.id == module
+                                    and any(name == shortname for name, _ in imp.names)
+                                    and submodule not in state.suppressed_set):
+                                state.suppressed.append(submodule)
+                                state.suppressed_set.add(submodule)
+    return messages
