@@ -467,6 +467,17 @@ class StrConv(NodeVisitor[str]):
             a += ['UpperBound({})'.format(o.upper_bound)]
         return self.dump(a, o)
 
+    def visit_paramspec_expr(self, o: 'mypy.nodes.ParamSpecExpr') -> str:
+        import mypy.types
+        a = []  # type: List[Any]
+        if o.variance == mypy.nodes.COVARIANT:
+            a += ['Variance(COVARIANT)']
+        if o.variance == mypy.nodes.CONTRAVARIANT:
+            a += ['Variance(CONTRAVARIANT)']
+        if not mypy.types.is_named_instance(o.upper_bound, 'builtins.object'):
+            a += ['UpperBound({})'.format(o.upper_bound)]
+        return self.dump(a, o)
+
     def visit_type_alias_expr(self, o: 'mypy.nodes.TypeAliasExpr') -> str:
         return 'TypeAliasExpr({})'.format(o.type)
 
@@ -553,7 +564,7 @@ def dump_tagged(nodes: Sequence[object], tag: Optional[str], str_conv: 'StrConv'
             a.append(indent(n.accept(str_conv), 2))
         elif isinstance(n, Type):
             a.append(indent(n.accept(TypeStrVisitor(str_conv.id_mapper)), 2))
-        elif n:
+        elif n is not None:
             a.append(indent(str(n), 2))
     if tag:
         a[-1] += ')'
