@@ -14,6 +14,7 @@ on specified sources.
 
 import os
 import re
+import shutil
 
 from typing import List, Dict, Any, Tuple, Union, cast
 
@@ -35,7 +36,7 @@ from mypy.dmypy_server import Server
 from mypy.config_parser import parse_config_file
 from mypy.find_sources import create_source_list
 
-import pytest  # type: ignore  # no pytest in typeshed
+import pytest
 
 # Set to True to perform (somewhat expensive) checks for duplicate AST nodes after merge
 CHECK_CONSISTENCY = False
@@ -158,7 +159,7 @@ class FineGrainedSuite(DataSuite):
             options.follow_imports = 'error'
 
         for name, _ in testcase.files:
-            if 'mypy.ini' in name:
+            if 'mypy.ini' in name or 'pyproject.toml' in name:
                 parse_config_file(options, lambda: None, name)
                 break
 
@@ -215,8 +216,13 @@ class FineGrainedSuite(DataSuite):
                 # Modify/create file
                 copy_and_fudge_mtime(op.source_path, op.target_path)
             else:
-                # Delete file
-                os.remove(op.path)
+                # Delete file/directory
+                if os.path.isdir(op.path):
+                    # Sanity check to avoid unexpected deletions
+                    assert op.path.startswith('tmp')
+                    shutil.rmtree(op.path)
+                else:
+                    os.remove(op.path)
         sources = self.parse_sources(main_src, step, options)
 
         if step <= num_regular_incremental_steps:
