@@ -445,7 +445,9 @@ class StubgenUtilSuite(unittest.TestCase):
 
         assert_equal(remove_misplaced_type_comments(original), dest)
 
-    def test_common_dir_prefix(self) -> None:
+    @unittest.skipIf(sys.platform == 'win32',
+                     'Tests building the paths common ancestor on *nix')
+    def test_common_dir_prefix_unix(self) -> None:
         assert common_dir_prefix([]) == '.'
         assert common_dir_prefix(['x.pyi']) == '.'
         assert common_dir_prefix(['./x.pyi']) == '.'
@@ -458,6 +460,26 @@ class StubgenUtilSuite(unittest.TestCase):
         assert common_dir_prefix(['foo/x.pyi', 'foo/bar/zar/y.pyi']) == 'foo'
         assert common_dir_prefix(['foo/bar/zar/x.pyi', 'foo/bar/y.pyi']) == 'foo/bar'
         assert common_dir_prefix(['foo/bar/x.pyi', 'foo/bar/zar/y.pyi']) == 'foo/bar'
+        assert common_dir_prefix([r'foo/bar\x.pyi']) == 'foo'
+        assert common_dir_prefix([r'foo\bar/x.pyi']) == r'foo\bar'
+
+    @unittest.skipIf(sys.platform != 'win32',
+                     'Tests building the paths common ancestor on Windows')
+    def test_common_dir_prefix_win(self) -> None:
+        assert common_dir_prefix(['x.pyi']) == '.'
+        assert common_dir_prefix([r'.\x.pyi']) == '.'
+        assert common_dir_prefix([r'foo\bar\x.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo\bar\x.pyi',
+                                  r'foo\bar\y.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo\bar\x.pyi', r'foo\y.pyi']) == 'foo'
+        assert common_dir_prefix([r'foo\x.pyi', r'foo\bar\y.pyi']) == 'foo'
+        assert common_dir_prefix([r'foo\bar\zar\x.pyi', r'foo\y.pyi']) == 'foo'
+        assert common_dir_prefix([r'foo\x.pyi', r'foo\bar\zar\y.pyi']) == 'foo'
+        assert common_dir_prefix([r'foo\bar\zar\x.pyi', r'foo\bar\y.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo\bar\x.pyi', r'foo\bar\zar\y.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo/bar\x.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo\bar/x.pyi']) == r'foo\bar'
+        assert common_dir_prefix([r'foo/bar/x.pyi']) == r'foo\bar'
 
 
 class StubgenHelpersSuite(unittest.TestCase):
@@ -603,6 +625,14 @@ class StubgenPythonSuite(DataSuite):
 self_arg = ArgSig(name='self')
 
 
+class TestBaseClass:
+    pass
+
+
+class TestClass(TestBaseClass):
+    pass
+
+
 class StubgencSuite(unittest.TestCase):
     """Unit tests for stub generation from C modules using introspection.
 
@@ -646,7 +676,7 @@ class StubgencSuite(unittest.TestCase):
         mod = ModuleType('module', '')  # any module is fine
         generate_c_type_stub(mod, 'C', TestClassVariableCls, output, imports)
         assert_equal(imports, [])
-        assert_equal(output, ['class C:', '    x: Any = ...'])
+        assert_equal(output, ['class C:', '    x: ClassVar[int] = ...'])
 
     def test_generate_c_type_inheritance(self) -> None:
         class TestClass(KeyError):
@@ -660,12 +690,6 @@ class StubgencSuite(unittest.TestCase):
         assert_equal(imports, [])
 
     def test_generate_c_type_inheritance_same_module(self) -> None:
-        class TestBaseClass:
-            pass
-
-        class TestClass(TestBaseClass):
-            pass
-
         output = []  # type: List[str]
         imports = []  # type: List[str]
         mod = ModuleType(TestBaseClass.__module__, '')
@@ -686,6 +710,16 @@ class StubgencSuite(unittest.TestCase):
         assert_equal(output, ['class C(argparse.Action): ...', ])
         assert_equal(imports, ['import argparse'])
 
+    def test_generate_c_type_inheritance_builtin_type(self) -> None:
+        class TestClass(type):
+            pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType('module', '')
+        generate_c_type_stub(mod, 'C', TestClass, output, imports)
+        assert_equal(output, ['class C(type): ...', ])
+        assert_equal(imports, [])
+
     def test_generate_c_type_with_docstring(self) -> None:
         class TestClass:
             def test(self, arg0: str) -> None:
@@ -791,9 +825,84 @@ class StubgencSuite(unittest.TestCase):
             attribute = property(get_attribute, doc="")
 
         output = []  # type: List[str]
-        generate_c_property_stub('attribute', TestClass.attribute, output, readonly=True)
+        generate_c_property_stub('attribute', TestClass.attribute, [], [], output, readonly=True)
         assert_equal(output, ['@property', 'def attribute(self) -> str: ...'])
 
+    def test_generate_c_type_with_single_arg_generic(self) -> None:
+        class TestClass:
+            def test(self, arg0: str) -> None:
+                """
+                test(self: TestClass, arg0: List[int])
+                """
+                pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType(TestClass.__module__, '')
+        generate_c_function_stub(mod, 'test', TestClass.test, output, imports,
+                                 self_var='self', class_name='TestClass')
+        assert_equal(output, ['def test(self, arg0: List[int]) -> Any: ...'])
+        assert_equal(imports, [])
+
+    def test_generate_c_type_with_double_arg_generic(self) -> None:
+        class TestClass:
+            def test(self, arg0: str) -> None:
+                """
+                test(self: TestClass, arg0: Dict[str, int])
+                """
+                pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType(TestClass.__module__, '')
+        generate_c_function_stub(mod, 'test', TestClass.test, output, imports,
+                                 self_var='self', class_name='TestClass')
+        assert_equal(output, ['def test(self, arg0: Dict[str,int]) -> Any: ...'])
+        assert_equal(imports, [])
+
+    def test_generate_c_type_with_nested_generic(self) -> None:
+        class TestClass:
+            def test(self, arg0: str) -> None:
+                """
+                test(self: TestClass, arg0: Dict[str, List[int]])
+                """
+                pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType(TestClass.__module__, '')
+        generate_c_function_stub(mod, 'test', TestClass.test, output, imports,
+                                 self_var='self', class_name='TestClass')
+        assert_equal(output, ['def test(self, arg0: Dict[str,List[int]]) -> Any: ...'])
+        assert_equal(imports, [])
+
+    def test_generate_c_type_with_generic_using_other_module_first(self) -> None:
+        class TestClass:
+            def test(self, arg0: str) -> None:
+                """
+                test(self: TestClass, arg0: Dict[argparse.Action, int])
+                """
+                pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType(TestClass.__module__, '')
+        generate_c_function_stub(mod, 'test', TestClass.test, output, imports,
+                                 self_var='self', class_name='TestClass')
+        assert_equal(output, ['def test(self, arg0: Dict[argparse.Action,int]) -> Any: ...'])
+        assert_equal(imports, ['import argparse'])
+
+    def test_generate_c_type_with_generic_using_other_module_last(self) -> None:
+        class TestClass:
+            def test(self, arg0: str) -> None:
+                """
+                test(self: TestClass, arg0: Dict[str, argparse.Action])
+                """
+                pass
+        output = []  # type: List[str]
+        imports = []  # type: List[str]
+        mod = ModuleType(TestClass.__module__, '')
+        generate_c_function_stub(mod, 'test', TestClass.test, output, imports,
+                                 self_var='self', class_name='TestClass')
+        assert_equal(output, ['def test(self, arg0: Dict[str,argparse.Action]) -> Any: ...'])
+        assert_equal(imports, ['import argparse'])
+
     def test_generate_c_type_with_overload_pybind11(self) -> None:
         class TestClass:
             def __init__(self, arg0: str) -> None:
