@@ -12,22 +12,23 @@ other modules refer to them.
 """
 
 from abc import abstractmethod
-from collections import OrderedDict
-from typing import Generic, TypeVar, cast, Any, List, Callable, Iterable, Optional, Set
-from mypy_extensions import trait
+from mypy.ordered_dict import OrderedDict
+from typing import Generic, TypeVar, cast, Any, List, Callable, Iterable, Optional, Set, Sequence
+from mypy_extensions import trait, mypyc_attr
 
 T = TypeVar('T')
 
 from mypy.types import (
-    Type, AnyType, CallableType, Overloaded, TupleType, TypedDictType, LiteralType,
+    Type, AnyType, CallableType, Overloaded, TupleType, TypeGuardType, TypedDictType, LiteralType,
     RawExpressionType, Instance, NoneType, TypeType,
-    UnionType, TypeVarType, PartialType, DeletedType, UninhabitedType, TypeVarDef,
+    UnionType, TypeVarType, PartialType, DeletedType, UninhabitedType, TypeVarLikeDef,
     UnboundType, ErasedType, StarType, EllipsisType, TypeList, CallableArgument,
     PlaceholderType, TypeAliasType, get_proper_type
 )
 
 
 @trait
+@mypyc_attr(allow_interpreted_subclasses=True)
 class TypeVisitor(Generic[T]):
     """Visitor class for types (Type subclasses).
 
@@ -102,8 +103,13 @@ class TypeVisitor(Generic[T]):
     def visit_type_alias_type(self, t: TypeAliasType) -> T:
         pass
 
+    @abstractmethod
+    def visit_type_guard_type(self, t: TypeGuardType) -> T:
+        pass
+
 
 @trait
+@mypyc_attr(allow_interpreted_subclasses=True)
 class SyntheticTypeVisitor(TypeVisitor[T]):
     """A TypeVisitor that also knows how to visit synthetic AST constructs.
 
@@ -134,7 +140,7 @@ class SyntheticTypeVisitor(TypeVisitor[T]):
         pass
 
 
-@trait
+@mypyc_attr(allow_interpreted_subclasses=True)
 class TypeTranslator(TypeVisitor[Type]):
     """Identity type transformation.
 
@@ -218,8 +224,11 @@ class TypeTranslator(TypeVisitor[Type]):
     def translate_types(self, types: Iterable[Type]) -> List[Type]:
         return [t.accept(self) for t in types]
 
+    def visit_type_guard_type(self, t: TypeGuardType) -> Type:
+        return TypeGuardType(t.type_guard.accept(self))
+
     def translate_variables(self,
-                            variables: List[TypeVarDef]) -> List[TypeVarDef]:
+                            variables: Sequence[TypeVarLikeDef]) -> Sequence[TypeVarLikeDef]:
         return variables
 
     def visit_overloaded(self, t: Overloaded) -> Type:
@@ -242,7 +251,7 @@ class TypeTranslator(TypeVisitor[Type]):
         pass
 
 
-@trait
+@mypyc_attr(allow_interpreted_subclasses=True)
 class TypeQuery(SyntheticTypeVisitor[T]):
     """Visitor for performing queries of types.
 
@@ -317,6 +326,9 @@ class TypeQuery(SyntheticTypeVisitor[T]):
     def visit_union_type(self, t: UnionType) -> T:
         return self.query_types(t.items)
 
+    def visit_type_guard_type(self, t: TypeGuardType) -> T:
+        return t.type_guard.accept(self)
+
     def visit_overloaded(self, t: Overloaded) -> T:
         return self.query_types(t.items())
 
