@@ -1,6 +1,6 @@
 """Code generation for native function bodies."""
 
-from typing import Union, Dict
+from typing import Union, Optional
 from typing_extensions import Final
 
 from mypyc.common import (
@@ -8,18 +8,19 @@ from mypyc.common import (
 )
 from mypyc.codegen.emit import Emitter
 from mypyc.ir.ops import (
-    OpVisitor, Goto, Branch, Return, Assign, LoadInt, LoadErrorValue, GetAttr, SetAttr,
+    OpVisitor, Goto, Branch, Return, Assign, Integer, LoadErrorValue, GetAttr, SetAttr,
     LoadStatic, InitStatic, TupleGet, TupleSet, Call, IncRef, DecRef, Box, Cast, Unbox,
-    BasicBlock, Value, MethodCall, PrimitiveOp, EmitterInterface, Unreachable, NAMESPACE_STATIC,
-    NAMESPACE_TYPE, NAMESPACE_MODULE, RaiseStandardError, CallC, LoadGlobal, Truncate,
-    BinaryIntOp, LoadMem, GetElementPtr, LoadAddress
+    BasicBlock, Value, MethodCall, Unreachable, NAMESPACE_STATIC, NAMESPACE_TYPE, NAMESPACE_MODULE,
+    RaiseStandardError, CallC, LoadGlobal, Truncate, IntOp, LoadMem, GetElementPtr,
+    LoadAddress, ComparisonOp, SetMem, Register, LoadLiteral, AssignMulti, KeepAlive
 )
 from mypyc.ir.rtypes import (
-    RType, RTuple, is_tagged, is_int32_rprimitive, is_int64_rprimitive, RStruct
+    RType, RTuple, RArray, is_tagged, is_int32_rprimitive, is_int64_rprimitive, RStruct,
+    is_pointer_rprimitive, is_int_rprimitive
 )
-from mypyc.ir.func_ir import FuncIR, FuncDecl, FUNC_STATICMETHOD, FUNC_CLASSMETHOD
+from mypyc.ir.func_ir import FuncIR, FuncDecl, FUNC_STATICMETHOD, FUNC_CLASSMETHOD, all_values
 from mypyc.ir.class_ir import ClassIR
-from mypyc.ir.const_int import find_constant_integer_registers
+from mypyc.ir.pprint import generate_names_for_ir
 
 # Whether to insert debug asserts for all error handling, to quickly
 # catch errors propagating without exceptions set.
@@ -46,40 +47,43 @@ def native_function_header(fn: FuncDecl, emitter: Emitter) -> str:
 def generate_native_function(fn: FuncIR,
                              emitter: Emitter,
                              source_path: str,
-                             module_name: str,
-                             optimize_int: bool = True) -> None:
-    if optimize_int:
-        const_int_regs = find_constant_integer_registers(fn.blocks)
-    else:
-        const_int_regs = {}
-    declarations = Emitter(emitter.context, fn.env)
-    body = Emitter(emitter.context, fn.env)
-    visitor = FunctionEmitterVisitor(body, declarations, source_path, module_name, const_int_regs)
+                             module_name: str) -> None:
+    declarations = Emitter(emitter.context)
+    names = generate_names_for_ir(fn.arg_regs, fn.blocks)
+    body = Emitter(emitter.context, names)
+    visitor = FunctionEmitterVisitor(body, declarations, source_path, module_name)
 
     declarations.emit_line('{} {{'.format(native_function_header(fn.decl, emitter)))
     body.indent()
 
-    for r, i in fn.env.indexes.items():
+    for r in all_values(fn.arg_regs, fn.blocks):
         if isinstance(r.type, RTuple):
             emitter.declare_tuple_struct(r.type)
-        if i < len(fn.args):
-            continue  # skip the arguments
+        if isinstance(r.type, RArray):
+            continue  # Special: declared on first assignment
+
+        if r in fn.arg_regs:
+            continue  # Skip the arguments
+
         ctype = emitter.ctype_spaced(r.type)
         init = ''
-        if r in fn.env.vars_needing_init:
-            init = ' = {}'.format(declarations.c_error_value(r.type))
-        if r.name not in const_int_regs:
-            declarations.emit_line('{ctype}{prefix}{name}{init};'.format(ctype=ctype,
-                                                                        prefix=REG_PREFIX,
-                                                                        name=r.name,
-                                                                        init=init))
+        declarations.emit_line('{ctype}{prefix}{name}{init};'.format(ctype=ctype,
+                                                                     prefix=REG_PREFIX,
+                                                                     name=names[r],
+                                                                     init=init))
 
     # Before we emit the blocks, give them all labels
-    for i, block in enumerate(fn.blocks):
+    blocks = fn.blocks
+    for i, block in enumerate(blocks):
         block.label = i
 
-    for block in fn.blocks:
+    for i in range(len(blocks)):
+        block = blocks[i]
+        next_block = None
+        if i + 1 < len(blocks):
+            next_block = blocks[i + 1]
         body.emit_label(block)
+        visitor.next_block = next_block
         for op in block.ops:
             op.accept(visitor)
 
@@ -89,48 +93,51 @@ def generate_native_function(fn: FuncIR,
     emitter.emit_from_emitter(body)
 
 
-class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
+class FunctionEmitterVisitor(OpVisitor[None]):
     def __init__(self,
                  emitter: Emitter,
                  declarations: Emitter,
                  source_path: str,
-                 module_name: str,
-                 const_int_regs: Dict[str, int]) -> None:
+                 module_name: str) -> None:
         self.emitter = emitter
         self.names = emitter.names
         self.declarations = declarations
-        self.env = self.emitter.env
         self.source_path = source_path
         self.module_name = module_name
-        self.const_int_regs = const_int_regs
+        self.literals = emitter.context.literals
+        self.next_block = None  # type: Optional[BasicBlock]
 
     def temp_name(self) -> str:
         return self.emitter.temp_name()
 
     def visit_goto(self, op: Goto) -> None:
-        self.emit_line('goto %s;' % self.label(op.label))
+        if op.label is not self.next_block:
+            self.emit_line('goto %s;' % self.label(op.label))
 
     def visit_branch(self, op: Branch) -> None:
-        neg = '!' if op.negated else ''
+        true, false = op.true, op.false
+        negated = op.negated
+        if true is self.next_block and op.traceback_entry is None:
+            # Switch true/false since it avoids an else block.
+            true, false = false, true
+            negated = not negated
 
+        neg = '!' if negated else ''
         cond = ''
-        if op.op == Branch.BOOL_EXPR:
-            expr_result = self.reg(op.left)  # right isn't used
+        if op.op == Branch.BOOL:
+            expr_result = self.reg(op.value)
             cond = '{}{}'.format(neg, expr_result)
-        elif op.op == Branch.NEG_INT_EXPR:
-            expr_result = self.reg(op.left)
-            cond = '{} < 0'.format(expr_result)
         elif op.op == Branch.IS_ERROR:
-            typ = op.left.type
-            compare = '!=' if op.negated else '=='
+            typ = op.value.type
+            compare = '!=' if negated else '=='
             if isinstance(typ, RTuple):
                 # TODO: What about empty tuple?
                 cond = self.emitter.tuple_undefined_check_cond(typ,
-                                                               self.reg(op.left),
+                                                               self.reg(op.value),
                                                                self.c_error_value,
                                                                compare)
             else:
-                cond = '{} {} {}'.format(self.reg(op.left),
+                cond = '{} {} {}'.format(self.reg(op.value),
                                          compare,
                                          self.c_error_value(typ))
         else:
@@ -140,30 +147,28 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
         if op.traceback_entry is not None or op.rare:
             cond = 'unlikely({})'.format(cond)
 
-        self.emit_line('if ({}) {{'.format(cond))
-
-        self.emit_traceback(op)
-
-        self.emit_lines(
-            'goto %s;' % self.label(op.true),
-            '} else',
-            '    goto %s;' % self.label(op.false)
-        )
+        if false is self.next_block:
+            if op.traceback_entry is None:
+                self.emit_line('if ({}) goto {};'.format(cond, self.label(true)))
+            else:
+                self.emit_line('if ({}) {{'.format(cond))
+                self.emit_traceback(op)
+                self.emit_lines(
+                    'goto %s;' % self.label(true),
+                    '}'
+                )
+        else:
+            self.emit_line('if ({}) {{'.format(cond))
+            self.emit_traceback(op)
+            self.emit_lines(
+                'goto %s;' % self.label(true),
+                '} else',
+                '    goto %s;' % self.label(false)
+            )
 
     def visit_return(self, op: Return) -> None:
-        regstr = self.reg(op.reg)
-        self.emit_line('return %s;' % regstr)
-
-    def visit_primitive_op(self, op: PrimitiveOp) -> None:
-        args = [self.reg(arg) for arg in op.args]
-        if not op.is_void:
-            dest = self.reg(op)
-        else:
-            # This will generate a C compile error if used. The reason for this
-            # is that we don't want to insert "assert dest is not None" checks
-            # everywhere.
-            dest = '<undefined dest>'
-        op.desc.emit(self, args, dest)
+        value_str = self.reg(op.value)
+        self.emit_line('return %s;' % value_str)
 
     def visit_tuple_set(self, op: TupleSet) -> None:
         dest = self.reg(op)
@@ -184,11 +189,17 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
         if dest != src:
             self.emit_line('%s = %s;' % (dest, src))
 
-    def visit_load_int(self, op: LoadInt) -> None:
-        if op.name in self.const_int_regs:
-            return
-        dest = self.reg(op)
-        self.emit_line('%s = %d;' % (dest, op.value))
+    def visit_assign_multi(self, op: AssignMulti) -> None:
+        typ = op.dest.type
+        assert isinstance(typ, RArray)
+        dest = self.reg(op.dest)
+        # RArray values can only be assigned to once, so we can always
+        # declare them on initialization.
+        self.emit_line('%s%s[%d] = {%s};' % (
+            self.emitter.ctype_spaced(typ.item_type),
+            dest,
+            len(op.src),
+            ', '.join(self.reg(s) for s in op.src)))
 
     def visit_load_error_value(self, op: LoadErrorValue) -> None:
         if isinstance(op.type, RTuple):
@@ -200,6 +211,19 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
             self.emit_line('%s = %s;' % (self.reg(op),
                                          self.c_error_value(op.type)))
 
+    def visit_load_literal(self, op: LoadLiteral) -> None:
+        index = self.literals.literal_index(op.value)
+        s = repr(op.value)
+        if not any(x in s for x in ('/*', '*/', '\0')):
+            ann = ' /* %s */' % s
+        else:
+            ann = ''
+        if not is_int_rprimitive(op.type):
+            self.emit_line('%s = CPyStatics[%d];%s' % (self.reg(op), index, ann))
+        else:
+            self.emit_line('%s = (CPyTagged)CPyStatics[%d] | 1;%s' % (
+                self.reg(op), index, ann))
+
     def get_attr_expr(self, obj: str, op: Union[GetAttr, SetAttr], decl_cl: ClassIR) -> str:
         """Generate attribute accessor for normal (non-property) access.
 
@@ -359,7 +383,6 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
         rtype = op.receiver_type
         class_ir = rtype.class_ir
         name = op.method
-        method_idx = rtype.method_index(name)
         method = rtype.class_ir.get_method(name)
         assert method is not None
 
@@ -382,6 +405,7 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
                 dest, lib, NATIVE_PREFIX, method.cname(self.names), args))
         else:
             # Call using vtable.
+            method_idx = rtype.method_index(name)
             self.emit_line('{}CPY_GET_METHOD{}({}, {}, {}, {}, {})({}); /* {} */'.format(
                 dest, version, obj, self.emitter.type_struct_name(rtype.class_ir),
                 method_idx, rtype.struct_name(self.names), mtype, args, op.method))
@@ -416,8 +440,8 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
                     'PyErr_SetString(PyExc_{}, "{}");'.format(op.class_name, message))
             elif isinstance(op.value, Value):
                 self.emitter.emit_line(
-                    'PyErr_SetObject(PyExc_{}, {}{});'.format(op.class_name, REG_PREFIX,
-                                                              op.value.name))
+                    'PyErr_SetObject(PyExc_{}, {});'.format(op.class_name,
+                                                            self.emitter.reg(op.value)))
             else:
                 assert False, 'op value type must be either str or Value'
         else:
@@ -447,14 +471,20 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
                 ann = ' /* %s */' % s
         self.emit_line('%s = %s;%s' % (dest, op.identifier, ann))
 
-    def visit_binary_int_op(self, op: BinaryIntOp) -> None:
+    def visit_int_op(self, op: IntOp) -> None:
+        dest = self.reg(op)
+        lhs = self.reg(op.lhs)
+        rhs = self.reg(op.rhs)
+        self.emit_line('%s = %s %s %s;' % (dest, lhs, op.op_str[op.op], rhs))
+
+    def visit_comparison_op(self, op: ComparisonOp) -> None:
         dest = self.reg(op)
         lhs = self.reg(op.lhs)
         rhs = self.reg(op.rhs)
         lhs_cast = ""
         rhs_cast = ""
-        signed_op = {BinaryIntOp.SLT, BinaryIntOp.SGT, BinaryIntOp.SLE, BinaryIntOp.SGE}
-        unsigned_op = {BinaryIntOp.ULT, BinaryIntOp.UGT, BinaryIntOp.ULE, BinaryIntOp.UGE}
+        signed_op = {ComparisonOp.SLT, ComparisonOp.SGT, ComparisonOp.SLE, ComparisonOp.SGE}
+        unsigned_op = {ComparisonOp.ULT, ComparisonOp.UGT, ComparisonOp.ULE, ComparisonOp.UGE}
         if op.op in signed_op:
             lhs_cast = self.emit_signed_int_cast(op.lhs.type)
             rhs_cast = self.emit_signed_int_cast(op.rhs.type)
@@ -471,6 +501,15 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
         type = self.ctype(op.type)
         self.emit_line('%s = *(%s *)%s;' % (dest, type, src))
 
+    def visit_set_mem(self, op: SetMem) -> None:
+        dest = self.reg(op.dest)
+        src = self.reg(op.src)
+        dest_type = self.ctype(op.dest_type)
+        # clang whines about self assignment (which we might generate
+        # for some casts), so don't emit it.
+        if dest != src:
+            self.emit_line('*(%s *)%s = %s;' % (dest_type, dest, src))
+
     def visit_get_element_ptr(self, op: GetElementPtr) -> None:
         dest = self.reg(op)
         src = self.reg(op.src)
@@ -483,7 +522,12 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
     def visit_load_address(self, op: LoadAddress) -> None:
         typ = op.type
         dest = self.reg(op)
-        self.emit_line('%s = (%s)&%s;' % (dest, typ._ctype, op.src))
+        src = self.reg(op.src) if isinstance(op.src, Register) else op.src
+        self.emit_line('%s = (%s)&%s;' % (dest, typ._ctype, src))
+
+    def visit_keep_alive(self, op: KeepAlive) -> None:
+        # This is a no-op.
+        pass
 
     # Helpers
 
@@ -491,8 +535,15 @@ class FunctionEmitterVisitor(OpVisitor[None], EmitterInterface):
         return self.emitter.label(label)
 
     def reg(self, reg: Value) -> str:
-        if reg.name in self.const_int_regs:
-            return str(self.const_int_regs[reg.name])
+        if isinstance(reg, Integer):
+            val = reg.value
+            if val == 0 and is_pointer_rprimitive(reg.type):
+                return "NULL"
+            s = str(val)
+            if val >= (1 << 31):
+                # Avoid overflowing signed 32-bit int
+                s += 'U'
+            return s
         else:
             return self.emitter.reg(reg)
 
