@@ -1,6 +1,7 @@
 """Transform class definitions from the mypy AST form to IR."""
 
-from typing import List, Optional
+from typing import List, Optional, Tuple
+from typing_extensions import Final
 
 from mypy.nodes import (
     ClassDef, FuncDef, OverloadedFuncDef, PassStmt, AssignmentStmt, NameExpr, StrExpr,
@@ -11,7 +12,7 @@ from mypyc.ir.ops import (
     BasicBlock, Branch, MethodCall, NAMESPACE_TYPE, LoadAddress
 )
 from mypyc.ir.rtypes import (
-    object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
+    RType, object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
     is_object_rprimitive, is_none_rprimitive
 )
 from mypyc.ir.func_ir import FuncDecl, FuncSignature
@@ -76,7 +77,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         dataclass_non_ext = None
         type_obj = None
 
-    attrs_to_cache = []  # type: List[Lvalue]
+    attrs_to_cache = []  # type: List[Tuple[Lvalue, RType]]
 
     for stmt in cdef.defs.body:
         if isinstance(stmt, OverloadedFuncDef) and stmt.is_property:
@@ -123,7 +124,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
             typ = builder.load_native_type_object(cdef.fullname)
             value = builder.accept(stmt.rvalue)
             builder.call_c(
-                py_setattr_op, [typ, builder.load_static_unicode(lvalue.name), value], stmt.line)
+                py_setattr_op, [typ, builder.load_str(lvalue.name), value], stmt.line)
             if builder.non_function_scope() and stmt.is_final_def:
                 builder.init_final_static(lvalue, value, cdef.name)
         elif isinstance(stmt, ExpressionStmt) and isinstance(stmt.expr, StrExpr):
@@ -150,7 +151,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         builder.call_c(dict_set_item_op,
                        [
                            builder.load_globals_dict(),
-                           builder.load_static_unicode(cdef.name),
+                           builder.load_str(cdef.name),
                            non_ext_class
                        ], cdef.line)
 
@@ -166,7 +167,7 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
         tp_bases = builder.new_tuple(bases, cdef.line)
     else:
         tp_bases = builder.add(LoadErrorValue(object_rprimitive, is_borrowed=True))
-    modname = builder.load_static_unicode(builder.module_name)
+    modname = builder.load_str(builder.module_name)
     template = builder.add(LoadStatic(object_rprimitive, cdef.name + "_template",
                                    builder.module_name, NAMESPACE_TYPE))
     # Create the class
@@ -181,7 +182,7 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
                      FuncSignature([], bool_rprimitive)), [], -1))
     # Populate a '__mypyc_attrs__' field containing the list of attrs
     builder.call_c(py_setattr_op, [
-        tp, builder.load_static_unicode('__mypyc_attrs__'),
+        tp, builder.load_str('__mypyc_attrs__'),
         create_mypyc_attrs_tuple(builder, builder.mapper.type_to_ir[cdef.info], cdef.line)],
         cdef.line)
 
@@ -192,31 +193,68 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
     builder.call_c(dict_set_item_op,
                 [
                     builder.load_globals_dict(),
-                    builder.load_static_unicode(cdef.name),
+                    builder.load_str(cdef.name),
                     tp,
                 ], cdef.line)
 
     return tp
 
 
+# Mypy uses these internally as base classes of TypedDict classes. These are
+# lies and don't have any runtime equivalent.
+MAGIC_TYPED_DICT_CLASSES = (
+    'typing._TypedDict',
+    'typing_extensions._TypedDict',
+)  # type: Final[Tuple[str, ...]]
+
+
 def populate_non_ext_bases(builder: IRBuilder, cdef: ClassDef) -> Value:
     """Create base class tuple of a non-extension class.
 
     The tuple is passed to the metaclass constructor.
     """
+    is_named_tuple = cdef.info.is_named_tuple
     ir = builder.mapper.type_to_ir[cdef.info]
     bases = []
     for cls in cdef.info.mro[1:]:
         if cls.fullname == 'builtins.object':
             continue
+        if is_named_tuple and cls.fullname in ('typing.Sequence', 'typing.Iterable'):
+            # HAX: Synthesized base classes added by mypy don't exist at runtime, so skip them.
+            #      This could break if they were added explicitly, though...
+            continue
         # Add the current class to the base classes list of concrete subclasses
         if cls in builder.mapper.type_to_ir:
             base_ir = builder.mapper.type_to_ir[cls]
             if base_ir.children is not None:
                 base_ir.children.append(ir)
 
-        base = builder.load_global_str(cls.name, cdef.line)
+        if cls.fullname in MAGIC_TYPED_DICT_CLASSES:
+            # HAX: Mypy internally represents TypedDict classes differently from what
+            #      should happen at runtime. Replace with something that works.
+            module = 'typing'
+            if builder.options.capi_version < (3, 9):
+                name = 'TypedDict'
+                if builder.options.capi_version < (3, 8):
+                    # TypedDict was added to typing in Python 3.8.
+                    module = 'typing_extensions'
+            else:
+                # In Python 3.9 TypedDict is not a real type.
+                name = '_TypedDict'
+            base = builder.get_module_attr(module, name, cdef.line)
+        elif is_named_tuple and cls.fullname == 'builtins.tuple':
+            if builder.options.capi_version < (3, 9):
+                name = 'NamedTuple'
+            else:
+                # This was changed in Python 3.9.
+                name = '_NamedTuple'
+            base = builder.get_module_attr('typing', name, cdef.line)
+        else:
+            base = builder.load_global_str(cls.name, cdef.line)
         bases.append(base)
+        if cls.fullname in MAGIC_TYPED_DICT_CLASSES:
+            # The remaining base classes are synthesized by mypy and should be ignored.
+            break
     return builder.new_tuple(bases, cdef.line)
 
 
@@ -225,6 +263,15 @@ def find_non_ext_metaclass(builder: IRBuilder, cdef: ClassDef, bases: Value) ->
     if cdef.metaclass:
         declared_metaclass = builder.accept(cdef.metaclass)
     else:
+        if cdef.info.typeddict_type is not None and builder.options.capi_version >= (3, 9):
+            # In Python 3.9, the metaclass for class-based TypedDict is typing._TypedDictMeta.
+            # We can't easily calculate it generically, so special case it.
+            return builder.get_module_attr('typing', '_TypedDictMeta', cdef.line)
+        elif cdef.info.is_named_tuple and builder.options.capi_version >= (3, 9):
+            # In Python 3.9, the metaclass for class-based NamedTuple is typing.NamedTupleMeta.
+            # We can't easily calculate it generically, so special case it.
+            return builder.get_module_attr('typing', 'NamedTupleMeta', cdef.line)
+
         declared_metaclass = builder.add(LoadAddress(type_object_op.type,
                                                      type_object_op.src, cdef.line))
 
@@ -242,7 +289,7 @@ def setup_non_ext_dict(builder: IRBuilder,
     # Check if the metaclass defines a __prepare__ method, and if so, call it.
     has_prepare = builder.call_c(py_hasattr_op,
                                 [metaclass,
-                                builder.load_static_unicode('__prepare__')], cdef.line)
+                                builder.load_str('__prepare__')], cdef.line)
 
     non_ext_dict = Register(dict_rprimitive)
 
@@ -250,7 +297,7 @@ def setup_non_ext_dict(builder: IRBuilder,
     builder.add_bool_branch(has_prepare, true_block, false_block)
 
     builder.activate_block(true_block)
-    cls_name = builder.load_static_unicode(cdef.name)
+    cls_name = builder.load_str(cdef.name)
     prepare_meth = builder.py_get_attr(metaclass, '__prepare__', cdef.line)
     prepare_dict = builder.py_call(prepare_meth, [cls_name, bases], cdef.line)
     builder.assign(non_ext_dict, prepare_dict, cdef.line)
@@ -269,7 +316,7 @@ def add_non_ext_class_attr(builder: IRBuilder,
                            lvalue: NameExpr,
                            stmt: AssignmentStmt,
                            cdef: ClassDef,
-                           attr_to_cache: List[Lvalue]) -> None:
+                           attr_to_cache: List[Tuple[Lvalue, RType]]) -> None:
     """Add a class attribute to __annotations__ of a non-extension class.
 
     If the attribute is initialized with a value, also add it to __dict__.
@@ -277,7 +324,7 @@ def add_non_ext_class_attr(builder: IRBuilder,
     # We populate __annotations__ because dataclasses uses it to determine
     # which attributes to compute on.
     # TODO: Maybe generate more precise types for annotations
-    key = builder.load_static_unicode(lvalue.name)
+    key = builder.load_str(lvalue.name)
     typ = builder.add(LoadAddress(type_object_op.type, type_object_op.src, stmt.line))
     builder.call_c(dict_set_item_op, [non_ext.anns, key, typ], stmt.line)
 
@@ -294,7 +341,8 @@ def add_non_ext_class_attr(builder: IRBuilder,
             # Skip "_order_" and "__order__", since Enum will remove it
             and lvalue.name not in ('_order_', '__order__')
         ):
-            attr_to_cache.append(lvalue)
+            # Enum values are always boxed, so use object_rprimitive.
+            attr_to_cache.append((lvalue, object_rprimitive))
 
 
 def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef) -> None:
@@ -389,7 +437,7 @@ def load_non_ext_class(builder: IRBuilder,
                        ir: ClassIR,
                        non_ext: NonExtClassInfo,
                        line: int) -> Value:
-    cls_name = builder.load_static_unicode(ir.name)
+    cls_name = builder.load_str(ir.name)
 
     finish_non_ext_dict(builder, non_ext, line)
 
@@ -419,27 +467,29 @@ def load_decorated_class(builder: IRBuilder, cdef: ClassDef, type_obj: Value) ->
     return dec_class
 
 
-def cache_class_attrs(builder: IRBuilder, attrs_to_cache: List[Lvalue], cdef: ClassDef) -> None:
+def cache_class_attrs(builder: IRBuilder,
+                      attrs_to_cache: List[Tuple[Lvalue, RType]],
+                      cdef: ClassDef) -> None:
     """Add class attributes to be cached to the global cache."""
-    typ = builder.load_native_type_object(cdef.fullname)
-    for lval in attrs_to_cache:
+    typ = builder.load_native_type_object(cdef.info.fullname)
+    for lval, rtype in attrs_to_cache:
         assert isinstance(lval, NameExpr)
         rval = builder.py_get_attr(typ, lval.name, cdef.line)
-        builder.init_final_static(lval, rval, cdef.name)
+        builder.init_final_static(lval, rval, cdef.name, type_override=rtype)
 
 
 def create_mypyc_attrs_tuple(builder: IRBuilder, ir: ClassIR, line: int) -> Value:
     attrs = [name for ancestor in ir.mro for name in ancestor.attributes]
     if ir.inherits_python:
         attrs.append('__dict__')
-    items = [builder.load_static_unicode(attr) for attr in attrs]
+    items = [builder.load_str(attr) for attr in attrs]
     return builder.new_tuple(items, line)
 
 
 def finish_non_ext_dict(builder: IRBuilder, non_ext: NonExtClassInfo, line: int) -> None:
     # Add __annotations__ to the class dict.
     builder.call_c(dict_set_item_op,
-                [non_ext.dict, builder.load_static_unicode('__annotations__'),
+                [non_ext.dict, builder.load_str('__annotations__'),
                 non_ext.anns], -1)
 
     # We add a __doc__ attribute so if the non-extension class is decorated with the
@@ -447,9 +497,9 @@ def finish_non_ext_dict(builder: IRBuilder, non_ext: NonExtClassInfo, line: int)
     # https://github.com/python/cpython/blob/3.7/Lib/dataclasses.py#L957
     filler_doc_str = 'mypyc filler docstring'
     builder.add_to_non_ext_dict(
-        non_ext, '__doc__', builder.load_static_unicode(filler_doc_str), line)
+        non_ext, '__doc__', builder.load_str(filler_doc_str), line)
     builder.add_to_non_ext_dict(
-        non_ext, '__module__', builder.load_static_unicode(builder.module_name), line)
+        non_ext, '__module__', builder.load_str(builder.module_name), line)
 
 
 def dataclass_finalize(
